/*
	Zilog Z80 Emulator Header
*/

#if !defined(Z80_H)
#define Z80_H

#include <stdio.h>
#if defined(Z80_USE_SDL)
#	include "SDL.h"
#	if SDL_BYTEORDER == SDL_LIL_ENDIAN
#		define Z80_LITTLEENDIAN	1
#	elif SDL_BYTEORDER == SDL_BIG_ENDIAN
#		define Z80_BIGENDIAN	1
#	endif
#endif

/* z80exec()̖߂l */
#define Z80_RUN	0	/* 쒆 */
#define Z80_HALT	1	/* HALT */
#define Z80_UNDERFLOW	2	/* X^bNA_[t[ */

#define Z80_STATES(z)	((z)->i.total_states - (z)->i.states)
#define Z80_RESET_STATES(z)	((z)->i.total_states = (z)->i.states)

typedef unsigned char	uint8;
typedef char	int8;
typedef unsigned short	uint16;
typedef short	int16;
typedef unsigned int	uint32;
typedef int	int32;

/* 8bitsWX^ */
typedef struct {
	uint8 *m;
#if defined(Z80_LITTLEENDIAN)
	uint8 f, a;	/* tO, AL[^ */
	uint8 c, b;	/* ėpWX^C, B */
	uint8 e, d;	/* ėpWX^E, D */
	uint8 l, h;	/* ėpWX^L, H */
	uint8 ixl, ixh;	/* CfbNXWX^IXl, IXh */
	uint8 iyl, iyh;	/* CfbNXWX^IYl, IYh */
	uint8 i;	/* C^vgWX^I */
	uint8 pad1;
	uint8 f_d, a_d;	/* ⏕WX^F', A' */
#elif defined(Z80_BIGENDIAN)
	uint8 a, f;	/* AL[^, tO */
	uint8 b, c;	/* ėpWX^B, C */
	uint8 d, e;	/* ėpWX^D, E */
	uint8 h, l;	/* ėpWX^H, L */
	uint8 ixh, ixl;	/* CfbNXWX^IXh, IXl */
	uint8 iyh, iyl;	/* CfbNXWX^IYh, IYl */
	uint8 pad2;
	uint8 i;	/* C^vgWX^I */
	uint8 a_d, f_d;	/* ⏕WX^A', F' */
#else
	ERROR: MUST DEFINE Z80_LITTLEENDIAN OR Z80_BIGENDIAN
#endif
	uint8 pad3, pad4;
	uint8 pad5, pad6;
	uint8 pad7, pad8;
	uint8 pad9, pad10;
	uint8 pad11, pad12;
	uint8 iff;	/* IFF1, IFF2 */
	uint8 im;	/* 荞݃[h */
	uint8 halt;	/* HALT? */
	uint8 pad13;
	uint8 pad14, pad15;
} Z80regs;

/* 16bitsWX^ */
typedef struct {
	uint8 *m;
	uint16 af;	/* yAWX^AF */
	uint16 bc;	/* yAWX^BC */
	uint16 de;	/* yAWX^DE */
	uint16 hl;	/* yAWX^HL */
	uint16 ix;	/* CfbNXWX^IX */
	uint16 iy;	/* CfbNXWX^IY */
	uint16 pad2;
	uint16 af_d;	/* ⏕WX^AF' */
	uint16 bc_d;	/* ⏕WX^BC' */
	uint16 de_d;	/* ⏕WX^DE' */
	uint16 hl_d;	/* ⏕WX^HL' */
	uint16 sp;	/* X^bN|C^SP */
	uint16 pc;	/* vOJE^PC */
} Z80regs16;

/* IvVȆ̏ */
typedef struct {
	Z80regs pad1;
	int states;	/* sXe[g */
	uint16 stack_under;	/* X^bN */
	uint16 pad2;
	int total_states;	/* ݐσXe[g */
	int trace;	/* g[X[h? */
	int emulate_subroutine;	/* Tu[`G~[g邩? */
	void *tag;	/* ̑̏ */
} Z80info;

/* WX^ */
typedef union {
	uint8     *m;	/*  */
	Z80regs   r;	/* 8bitsWX^ */
	Z80regs16 r16;	/* 16bitsWX^ */
	Z80info   i;	/* IvVȆ̏ */
} Z80stat;

/* z80.c */
void z80srand(uint32);
int z80reset(Z80stat *);
int z80nmi(Z80stat *);
int z80int0chk(const Z80stat *);
int z80int0(Z80stat *, uint8);
int z80int1chk(const Z80stat *);
int z80int1(Z80stat *);
int z80int2chk(const Z80stat *);
int z80int2(Z80stat *, uint8);
int z80exec(Z80stat *);

/* z80disasm.c */
void *z80disasm(char *, uint8 *);
char *z80regs(char *, const Z80stat *);

#include "z80memory.h"
int z80inport(Z80stat *, uint8 *, uint8);
int z80outport(Z80stat *, uint8, uint8);
int z80subroutine(Z80stat *, uint16);
void z80log(const Z80stat *);

#endif

/*
	Copyright 2005 ~ 2013 maruhiro
	All rights reserved. 

	Redistribution and use in source and binary forms, 
	with or without modification, are permitted provided that 
	the following conditions are met: 

	 1. Redistributions of source code must retain the above copyright notice, 
	    this list of conditions and the following disclaimer. 

	 2. Redistributions in binary form must reproduce the above copyright notice, 
	    this list of conditions and the following disclaimer in the documentation 
	    and/or other materials provided with the distribution. 

	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
	FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
	THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
	ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* eof */
